#include "StdAfx.h"
#include "AppProfile.h"

#define DEF_AND_KEY(x)	x##_DEF,x##_KEY

#define PROFILE_INIT_SEC		_T("Initialize")
#define PROFILE_PROCESS_SEC		_T("IgnoreProcess")
#define PROFILE_CLASS_SEC		_T("IgnoreClass")
#define PROFILE_DISABLE1ST_SEC	_T("DisableFirst")
#define PROFILE_VKKANA_DEF		VK_KANA
#define PROFILE_VKKANA_KEY		_T("vkKana")

#define PROFILE_KANA_DEF		0
#define PROFILE_KANA_KEY		_T("HiraganaKANA")
#define PROFILE_ATOKTAB_DEF		0
#define PROFILE_ATOKTAB_KEY		_T("ATOK")
#define PROFILE_CTRL1_DEF		0
#define PROFILE_CTRL1_KEY		_T("Ctrl1")
#define PROFILE_BOOTCAMP_DEF	0
#define PROFILE_BOOTCAMP_KEY	_T("Bootcamp")
#define PROFILE_VJED4_DEF		0
#define PROFILE_VJED4_KEY		_T("VJEDelta4")
#define PROFILE_UNLOAD_DEF		0
#define PROFILE_UNLOAD_KEY		_T("UnloadDll")
#define PROFILE_DISABLE1ST_DEF	0
#define PROFILE_DISABLE1ST_KEY	_T("DisableFirstIME")

typedef struct _DEF_AND_KEY_ARRAY {
	BOOL def;
	TCHAR key[20];
} DEF_AND_KEY_ARRAY;
static const DEF_AND_KEY_ARRAY defArray[sizeof(_SETTINGS::b_uni::b_st)/sizeof(BOOL)] = {
	{DEF_AND_KEY(PROFILE_KANA)},
	{DEF_AND_KEY(PROFILE_ATOKTAB)},
	{DEF_AND_KEY(PROFILE_CTRL1)},
	{DEF_AND_KEY(PROFILE_BOOTCAMP)},
	{DEF_AND_KEY(PROFILE_VJED4)},
	{DEF_AND_KEY(PROFILE_UNLOAD)},
	{DEF_AND_KEY(PROFILE_DISABLE1ST)},
};

AppProfile::AppProfile(void)
	: PrivateProfile( _T(".ini") )
	, set(NULL)
{
}

AppProfile::~AppProfile(void)
{
}

BOOL AppProfile::SetAddress(SETTINGS *p)
{
	if ( p == NULL ) {
		return FALSE;
	}
	set = p;
	return TRUE;
}

DWORD AppProfile::GetSize(void)
{
	return sizeof(SETTINGS) + FlexReadSection( PROFILE_PROCESS_SEC, NULL ) + FlexReadSection( PROFILE_CLASS_SEC, NULL ) + ReadDisable1stSection( PROFILE_DISABLE1ST_SEC, NULL );
}

BOOL AppProfile::Load(void)
{
	if ( set == NULL ) {
		return FALSE;
	}

	for ( int c = 0; c < _countof(defArray); c++ ) {
		set->b.boolArray[c] = ReadInt( defArray[c].def, defArray[c].key, PROFILE_INIT_SEC );
	}

	set->vkKana = ReadHex( DEF_AND_KEY( PROFILE_VKKANA ),	PROFILE_INIT_SEC );

	if ( ( set->dwSizeProcessName = FlexReadSection( PROFILE_PROCESS_SEC, NULL ) ) != 0 ) {
		TCHAR *temp = NULL;
		FlexReadSection( PROFILE_PROCESS_SEC, &temp );
		memcpy( (LPBYTE)set + sizeof(SETTINGS), temp, set->dwSizeProcessName );
		delete[] temp;
	}

	if ( ( set->dwSizeClassName = FlexReadSection( PROFILE_CLASS_SEC, NULL ) ) != 0 ) {
		TCHAR *temp = NULL;
		FlexReadSection( PROFILE_CLASS_SEC, &temp );
		memcpy( (LPBYTE)set + sizeof(SETTINGS) + set->dwSizeProcessName, temp, set->dwSizeClassName );
		delete[] temp;
	}

	if ( ( set->dwSizeDisableFirst = ReadDisable1stSection( PROFILE_DISABLE1ST_SEC, NULL ) ) != 0 ) {
		BYTE *temp = NULL;
		ReadDisable1stSection( PROFILE_DISABLE1ST_SEC, &temp );
		memcpy( (LPBYTE)set + sizeof(SETTINGS) + set->dwSizeProcessName + set->dwSizeClassName, temp, set->dwSizeDisableFirst );
		delete[] temp;
	}

	return TRUE;
}

BOOL AppProfile::Save(void)
{
	for ( int c = 0; c < _countof(defArray); c++ ) {
		WriteInt( set->b.boolArray[c], defArray[c].def, defArray[c].key, PROFILE_INIT_SEC );
	}

	return TRUE;
}

DWORD AppProfile::ReadDisable1stSection( const TCHAR *section, BYTE **value )
{
	TCHAR *temp = NULL, *p;
	BYTE *d = NULL;
	DWORD msg, cnt;
	DWORD len = 0, total = 0, unit, namelen;

	FlexReadSection( PROFILE_DISABLE1ST_SEC, &temp );

	if ( value ) {
		*value = (BYTE*)malloc( 0 );
		d = *value;
	}

	p = temp;
	while( *p != 0 ) {
		if ( _stscanf_s( p, _T("%d"), &msg ) != 1 ) goto next;
		p = _tcschr( p, _T(',') );
		if ( p == NULL ) goto next;
		p++;
		if ( _stscanf_s( p, _T("%d"), &cnt ) != 1 ) goto next;
		p = _tcschr( p, _T(',') );
		if ( p == NULL ) goto next;
		p++;
		if ( *p == 0 ) goto next;

		unit = sizeof(DWORD) * 2;
		namelen = (DWORD)_tcslen( p ) + 1;
		unit += namelen * sizeof(TCHAR);
		len += unit;
		total++;

		if ( value ) {
			*value = (BYTE*)realloc( *value, len );
			d = *value + len - unit;
			DISABLEFIRST *ds = (DISABLEFIRST*)d;
			ds->wParam = msg;
			ds->count = cnt;
			_tcscpy_s( ds->name, namelen, p );
		}

next:
		p += _tcslen( p ) + 1;
	}

	if ( total > 0 ) {
		unit = sizeof(DISABLEFIRST);
		len += unit;
		if ( value ) {
			*value = (BYTE*)realloc( *value, len );
			d = *value + len - unit;
			DISABLEFIRST *ds = (DISABLEFIRST*)d;
			memset( ds, 0, unit );
		}
	}

	delete[] temp;

	return len;
}
