#include "stdafx.h"
#include "DynamicApi.h"
#include "crtdbg.h"
#include "common.h"

#define GetEachFunc(x,y,z)	if(x==NULL)x=(y)GetFuncAddress(z)
#define GetEachFunc2(x,y)	if(p##x==NULL)p##x=(y)GetFuncAddress(#x)
#define GetEachFunc3(x)		if(p##x==NULL)p##x=(DLL##x)GetFuncAddress(#x)
#define InvokeFunc(x,y,...)	p##x?p##x(__VA_ARGS__):y

DeltaEndApi deltaDll;
Kernel32Api kernel32;
PsapiApi psapi;
User32Api user32;
Shell32Api shell32;
Imm32Api imm32;



DynamicApi::DynamicApi( const TCHAR *name, BOOL notLoad, DLL_DIR dir, const TCHAR *path )
	: dllHandle( NULL )
	, notFreeLib( notLoad )
{
	switch( dir )
	{
	case SYSTEM32:
		_ASSERT( name != NULL );
		GetSystemDirectory( dllPath, _countof(dllPath ) );
		_tcscat_s( dllPath, _countof(dllPath), _T("\\") );
		break;

	case EXE:
		_ASSERT( name != NULL );
		GetModuleFileName( NULL, dllPath, _countof(dllPath) );
		*( _tcsrchr( dllPath, _T('\\') ) + 1 ) = 0;
		break;

	case CURRENT:
		_ASSERT( name != NULL );
		GetCurrentDirectory( _countof(dllPath ), dllPath );
		_tcscat_s( dllPath, _countof(dllPath), _T("\\") );
		break;

	case SPECIFY:
		_ASSERT( name != NULL );
		_ASSERT( path != NULL );
		if ( path ) {
			_tcscpy_s( dllPath, _countof(dllPath), path );
			if ( dllPath[ _tcslen(dllPath) - 1 ] != _T('\\') ) {
				_tcscat_s( dllPath, _countof(dllPath), _T("\\") );
			}
		}
		else {
			GetSystemDirectory( dllPath, _countof(dllPath ) );
			_tcscat_s( dllPath, _countof(dllPath), _T("\\") );
		}
		break;

	case SEARCH:
		_ASSERT( name != NULL );
		dllPath[0] = 0;
		break;

	case AUTO3264:
		_ASSERT( name == NULL );
		GetModuleFileName( NULL, dllPath, _countof(dllPath) );
		{
			TCHAR *p = _tcsrchr( dllPath, _T('\\') ) + 1;
			TCHAR *e = _tcschr( p, _T('.') );
			if ( e ) *e = 0;
			if ( ( _tcslen( p ) > 2 ) && ( _tcscmp( ( e = p + _tcslen(p) - 2 ), _T("64") ) == 0 ) ) {
					*e = 0;
					_tcscat_s( dllPath, _countof(dllPath), _T("Dll64") );
			}
			else {
				_tcscat_s( dllPath, _countof(dllPath), _T("Dll") );
			}
			_tcscat_s( dllPath, _countof(dllPath), _T(".dll") );
		}
		break;
	}
	if ( name ) _tcscat_s( dllPath, _countof(dllPath), name );
}

DynamicApi::~DynamicApi(void)
{
	if ( !notFreeLib ) {
		FreeLibrary( dllHandle );
		dllHandle = NULL;
	}
}

BOOL DynamicApi::Init(void)
{
	if ( dllHandle == NULL ) {
		if ( notFreeLib ) {
			dllHandle = GetModuleHandle( dllPath );
			if ( dllHandle == NULL ) {
				notFreeLib = FALSE;
				dllHandle = LoadLibrary( dllPath );
			}
		}
		else {
			dllHandle = LoadLibrary( dllPath );
		}
	}

	if ( _tcschr( dllPath, _T('\\') ) == NULL ) {
		GetModuleFileName( dllHandle, dllPath, _countof(dllPath) );
	}

	return dllHandle ? TRUE : FALSE;
}

FARPROC DynamicApi::GetFuncAddress( const char *func )
{
	if ( dllHandle == NULL ) Init();
	return GetProcAddress( dllHandle, func );
}



BOOL DeltaEndApi::SetHook( HWND hWnd )
{
	GetEachFunc3( SetHook );
	return ( hooked = InvokeFunc( SetHook, FALSE, hWnd ) );
}

BOOL DeltaEndApi::ReleaseHook( void )
{
	BOOL bRet = FALSE;

	if ( hooked ) {
		GetEachFunc3( ReleaseHook );
		bRet = InvokeFunc( ReleaseHook, FALSE );
		hooked = FALSE;
	}
	return bRet;
}



BOOL Kernel32Api::IsWow64Process( HANDLE hProcess, PBOOL Wow64Process )
{
	GetEachFunc3( IsWow64Process );

	if ( pIsWow64Process ) {
		pIsWow64Process( hProcess, Wow64Process );
	}
	else {
		*Wow64Process = FALSE;
	}
	return TRUE;
}

BOOL Kernel32Api::GetNativeSystemInfo( LPSYSTEM_INFO lpSystemInfo )
{
	GetEachFunc3( GetNativeSystemInfo );
	InvokeFunc( GetNativeSystemInfo, FALSE, lpSystemInfo );

	return pGetNativeSystemInfo ? TRUE : FALSE;
}



DWORD PsapiApi::GetModuleFileNameEx( HANDLE hProcess, HMODULE hModule, LPTSTR lpFilename, DWORD nSize )
{
	const char *funcName;
#ifdef _UNICODE
	funcName = "GetModuleFileNameExW";
#else
	funcName = "GetModuleFileNameExA";
#endif
	GetEachFunc( pGetModuleFileNameEx, DLLGetModuleFileNameEx, funcName );
	return InvokeFunc( GetModuleFileNameEx, 0, hProcess, hModule, lpFilename, nSize );
}

BOOL PsapiApi::EnumProcesses( DWORD* pProcessIds, DWORD cb, DWORD* pBytesReturned )
{
	GetEachFunc3( EnumProcesses );
	return InvokeFunc( EnumProcesses, FALSE, pProcessIds, cb, pBytesReturned );
}

BOOL PsapiApi::EnumProcessModules( HANDLE hProcess, HMODULE* lphModule, DWORD cb, LPDWORD lpcbNeeded )
{
	GetEachFunc3( EnumProcessModules );
	return InvokeFunc( EnumProcessModules, FALSE, hProcess, lphModule, cb, lpcbNeeded );
}

BOOL PsapiApi::GetModuleInformation( HANDLE hProcess, HMODULE hModule, LPMODULEINFO lpmodinfo, DWORD cb )
{
	GetEachFunc3( GetModuleInformation );
	return InvokeFunc( GetModuleInformation, FALSE, hProcess, hModule, lpmodinfo, cb );
}

DWORD PsapiApi::GetProcessImageFileName( HANDLE hProcess, LPTSTR lpImageFileName, DWORD nSize )
{
	const char *funcName;
#ifdef _UNICODE
	funcName = "GetProcessImageFileNameW";
#else
	funcName = "GetProcessImageFileNameA";
#endif
	GetEachFunc( pGetProcessImageFileName, DLLGetProcessImageFileName, funcName );
	return InvokeFunc( GetProcessImageFileName, 0, hProcess, lpImageFileName, nSize );
}



BOOL User32Api::ChangeWindowMessageFilter( UINT message, DWORD dwFlag )
{
	GetEachFunc3( ChangeWindowMessageFilter );
	return InvokeFunc( ChangeWindowMessageFilter, FALSE, message, dwFlag );
}



Shell32Api::Shell32Api(void)
	: DynamicApi( _T("shell32.dll") )
	, pDllGetVersion( NULL )
	, oldVersion( FALSE )
{
	if ( GetSimpleOSVersion() < OS_2000 ) oldVersion = TRUE;
}

HRESULT Shell32Api::DllGetVersion( DLLVERSIONINFO *pdvi )
{
	GetEachFunc2( DllGetVersion, DLLGETVERSIONPROC );
	return InvokeFunc( DllGetVersion, E_NOTIMPL, pdvi );
}



BOOL Imm32Api::ImmDisableIME( DWORD idThread )
{
	GetEachFunc3( ImmDisableIME );
	return InvokeFunc( ImmDisableIME, FALSE, idThread );
}



NTSTATUS NtDllApi::LdrGetProcedureAddress( HMODULE moduleHandle, PANSI_STRING functionName, WORD ordinalNumber, PVOID functionAddress )
{
	GetEachFunc3( LdrGetProcedureAddress );
	return InvokeFunc( LdrGetProcedureAddress, STATUS_NOT_IMPLEMENTED, moduleHandle, functionName, ordinalNumber, functionAddress );
}

VOID NtDllApi::RtlInitAnsiString( PANSI_STRING DestinationString, const char *SourceString )
{
	GetEachFunc3( RtlInitAnsiString );
	InvokeFunc( RtlInitAnsiString, NULL, DestinationString, SourceString );
}
