#include "StdAfx.h"
#include "resource.h"
#include "MainWindow.h"
#include "misc.h"
#include "AboutDlg.h"
#include "common.h"
#include "DynamicApi.h"
#include <windowsx.h>

SETTINGS *shared;
UINT wmDeltaEnd;

BOOL KbSetHook( HWND hWnd );
BOOL KbReleaseHook(void);
DWORD UnloadWndProcHookDll( const TCHAR *dllNameToUnload );



MainWindow::MainWindow(void)
	: m_aboutDlg( NULL )
	, waitForMsgBox( FALSE )
	, m_imeLoggerDlg( NULL )
{
	m_lpszClass = _T("MainWindow");
	m_idIcon = IDI_ICON_APP;
	m_hbrBackground = NULL;
	m_style = WS_POPUP;
	m_exStyle = WS_EX_TOOLWINDOW;
}

MainWindow::~MainWindow(void)
{
	UnregisterClass( szTitle, hInst );
}

INT_PTR CALLBACK MainWindow::WndProc( UINT message, WPARAM wParam, LPARAM lParam )
{
	switch (message)
	{
	HANDLE_MSG( m_hWnd, WM_CREATE,	Cls_OnCreate );
	HANDLE_MSG( m_hWnd, WM_COMMAND,	Cls_OnCommand );
	HANDLE_MSG( m_hWnd, WM_DESTROY,	Cls_OnDestroy );

	case WM_SHARED_STATUS:		Cls_OnSharedStatus( wParam, lParam );					break;
	case WM_FROMHOOK:			Cls_OnDllMessage( (UINT)wParam );						break;
	case WM_TRAYICONMESSAGE:	Cls_OnTrayIconMessage( (UINT)lParam, (UINT)wParam );	break;
	case WM_ENDSESSION:			PostMessage( m_hWnd, WM_CLOSE, 0, 0 );					break;
	case WM_CLOSE:				sync.CloseApp( m_hWnd );								break;

	default:
		if ( message == WM_TASKBARCREATED ) Cls_OnTaskbarCreated();
		return DefWindowProc(m_hWnd, message, wParam, lParam);
	}
	return 0;
}

void MainWindow::AddBootcampIcon(void)
{
	if ( IsNativePlatform() && psapi.Init() ) {
		EnumWindows( EnumBootcampProc, (LPARAM)this );
	}
}

BOOL CALLBACK MainWindow::EnumBootcampProc(HWND hWnd, LPARAM lParam)
{
	TCHAR path[_MAX_PATH], fname[_MAX_FNAME];

	DWORD dwProcessId;
	HANDLE hHandle;
	HMODULE hProcess;

	GetWindowThreadProcessId(hWnd, &dwProcessId);

	hHandle = OpenProcess( PROCESS_ALL_ACCESS, TRUE, dwProcessId );

	hProcess = (HMODULE)(LONG_PTR)GetWindowLongPtr( hWnd, GWLP_HINSTANCE );

	path[0] = 0;
	DWORD dwErr = psapi.GetModuleFileNameEx( hHandle, hProcess, path, _MAX_PATH );

	_tsplitpath_s( path, NULL, 0, NULL, 0, fname, _MAX_FNAME, NULL, 0 );

	if ( ( _tcsicmp( fname, _T("bootcamp") ) == 0 ) || ( _tcsicmp( fname, _T("kbdmgr") ) == 0 ) ) {

		RECT rect;
		GetWindowRect( hWnd, &rect );
		if ( ( rect.right == 10 ) && ( rect.bottom == 10 ) ) {

			HMODULE hmod = LoadLibrary( path );
			((MainWindow*)lParam)->notifyIcon.AddIcon( hWnd, _T("Boot Camp"), 128, 0x86, WM_USER+10, hmod );
			FreeLibrary( hmod );

			return FALSE;
		}
	}

	CloseHandle( hHandle );
	return TRUE;
}

void MainWindow::Cls_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch ( id )
	{
	case ID_POPUP_KANA:
	case ID_POPUP_TAB:
	case ID_POPUP_CTRL:
	case ID_POPUP_BOOTCAMP:
	case ID_POPUP_VJE:
	case ID_POPUP_UNLOAD:
	case ID_POPUP_DISABLEFIRST:
		if ( !sync.IsSlaveMode() ) {
			prof.SetBools( id - ID_POPUP_KANA, prof.settings().b.boolArray[ id - ID_POPUP_KANA ] ? FALSE : TRUE );
		}
		break;

	case ID_POPUP_VER:
		if ( !sync.IsSlaveMode() ) {
			if ( m_aboutDlg ) {
				m_aboutDlg->SetFocus();
			}
			else {
				m_aboutDlg = new AboutDlg(IDD_ABOUTBOX);
				m_aboutDlg->SetSyncFlag( sync.GetRunMode() == RUN_MODE_MASTER );
				m_aboutDlg->DoModal(m_hWnd);
				delete m_aboutDlg;
				m_aboutDlg = NULL;
			}
		}
		break;

	case ID_POPUP_IMELOGGER:
		if ( !sync.IsSlaveMode() ) {
			if ( m_imeLoggerDlg ) {
				m_imeLoggerDlg->SetFocus();
			}
			else {
				m_imeLoggerDlg = new ImeLoggerDlg(IDD_IMELOGGER);
				m_imeLoggerDlg->DoModal(m_hWnd);
				delete m_imeLoggerDlg;
				m_imeLoggerDlg = NULL;
			}
		}
		break;

	case ID_POPUP_EXIT:
		PostMessage( m_hWnd, WM_CLOSE, 0, 0 );
		break;

	default:
		DefWindowProc( hwnd, WM_COMMAND, MAKEWPARAM( id, codeNotify), (LPARAM)hwndCtl );
	}
}

BOOL MainWindow::Cls_OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct)
{
	DWORD_PTR dwResult;
	TCHAR temp[256];

	do {
		if ( !notifyIcon.Init() ) {
			MessageBox( m_hWnd, _T("ʒm̈p̃bZ[WtB^[ݒł܂ł"), szTitle, MB_OK | MB_ICONEXCLAMATION );
			break;
		}

		wmDeltaEnd = RegisterWindowMessage( COMMON_MESSAGE_IDENTIFIER );
		if ( wmDeltaEnd == 0 ) {
			MessageBox( m_hWnd, _T("u[hLXgpbZ[W̓o^Ɏs܂"), szTitle, MB_OK | MB_ICONEXCLAMATION );
			break;
		}

		dwResult = sync.Init( m_hWnd, SHARED_MEMORY_IDENTIFIER, prof.GetSize(), (void**)&shared );
		if ( shared == NULL ) {
			_stprintf_s( temp, _countof(temp), _T("L̏Ɏs܂ : %d"), dwResult );
			MessageBox( m_hWnd, temp, szTitle, MB_OK | MB_ICONEXCLAMATION );
			DestroyWindow( m_hWnd );
			break;
		}

		if ( !deltaDll.Init() ) {
			MessageBox( m_hWnd, _T("DLLǂݍ߂܂ł"), szTitle, MB_OK | MB_ICONEXCLAMATION );
			PostMessage( m_hWnd, WM_CLOSE, 0, 0 );
			break;
		}

		// X[ȕꍇWM_SHARED_STATUS/STATUS_ACCEPTED󂯂Ďc̏
		if ( !sync.IsSlaveMode() ) {
			Init();
			user32.ChangeWindowMessageFilter( WM_FROMHOOK, MSGFLT_ADD );
		}
	} while ( FALSE );

	return TRUE;
}

void MainWindow::Init(void)
{
	DWORD_PTR dwResult;

	do {
		prof.SetAddress( shared );
		if ( !sync.IsSlaveMode() ) {
			prof.Load();
			if ( !KbSetHook( m_hWnd ) ) {
				MessageBox( m_hWnd, _T("L[{[htbNݒł܂ł"), szTitle, MB_OK | MB_ICONEXCLAMATION );
				PostMessage( m_hWnd, WM_CLOSE, 0, 0 );
				break;
			}
			if ( !notifyIcon.AddIcon( m_hWnd, szTitle, GetKeyState( VK_KANA ) & 1 ? IDI_ICON_ON : IDI_ICON_APP ) ) {
				MessageBox( m_hWnd, _T("ʒm̈ACRo^ł܂ł"), szTitle, MB_OK | MB_ICONEXCLAMATION );
				PostMessage( m_hWnd, WM_CLOSE, 0, 0 );
				break;
			}
		}
		if ( !deltaDll.SetHook( m_hWnd ) ) {
			MessageBox( m_hWnd, _T("bZ[WtbNݒł܂ł"), szTitle, MB_OK | MB_ICONEXCLAMATION );
			PostMessage( m_hWnd, WM_CLOSE, 0, 0 );
			break;
		}
		if ( shared->b.m.bVjed4 ) {
			//SendMessage( GetForegroundWindow(), wmDeltaEnd, 0, 0 );
			SendMessageTimeout( GetForegroundWindow(), wmDeltaEnd, 0, 0, SMTO_ABORTIFHUNG | SMTO_NORMAL, 1000, &dwResult );
		}
	} while( FALSE );
}

BOOL MainWindow::Cls_OnTrayIconMessage( UINT mouseMsg, UINT iconId )
{
	HMENU hMenu, hPopup;
	POINT pt;

	switch( mouseMsg )
	{
	case WM_RBUTTONUP:
		if ( GetSimpleOSVersion() >= OS_VISTA ) break;
	case WM_CONTEXTMENU:
		SetForegroundWindow( m_hWnd );

		hMenu = LoadMenu( hInst, MAKEINTRESOURCE(IDR_MENU1) );
		hPopup = GetSubMenu( hMenu, 0 );

		//for ( int c = 0; c < sizeof(_SETTINGS::b_uni::b_st) / sizeof(BOOL); c++ ) {
		for ( int c = 0; c < _countof(prof.settings().b.boolArray); c++ ) {
			CheckMenuItem( hPopup, ID_POPUP_KANA + c, prof.settings().b.boolArray[c] ? MF_CHECKED : MF_UNCHECKED );
		}

		EnableMenuItem( hPopup, ID_POPUP_UNLOAD, psapi.Init() ? MF_ENABLED : MF_GRAYED );

		GetCursorPos( &pt );
		TrackPopupMenu( hPopup, TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, NULL, m_hWnd, NULL );

		DestroyMenu( hMenu );
		return TRUE;
		break;
	}

	return FALSE;
}

void MainWindow::Cls_OnDestroy(HWND hwnd)
{
	shared = NULL;

	// tbN͕K̂ŁAɗpĂ
	if ( deltaDll.IsHooked() ) {
		deltaDll.ReleaseHook();
		if ( !sync.IsSlaveMode() ) {
			notifyIcon.DeleteIcon( m_hWnd );
			KbReleaseHook();
			if ( sync.GetRunMode() != RUN_MODE_FALLEN ) {
				prof.Save();
			}
			PostCompleteBroadcast();
		}
		if ( psapi.Init() && prof.bUnload() ) {
			Sleep( 100 );
			UnloadWndProcHookDll( deltaDll.GetPath() );
		}
	}
	PostQuitMessage(0);
}

void MainWindow::Cls_OnTaskbarCreated( void )
{
	if ( !sync.IsSlaveMode() ) {
		notifyIcon.AddIcon( m_hWnd, szTitle, GetKeyState( VK_KANA ) & 1 ? IDI_ICON_ON : IDI_ICON_APP );
	}
	// tbN͕K̂ŁAɗpĂ
	if ( deltaDll.IsHooked() && prof.bBootcamp() ) {
		AddBootcampIcon();
	}
}

void MainWindow::Cls_OnDllMessage( UINT type )
{
	DWORD_PTR dwResult;

	if ( type == 0 ) {	// JiL[ꂽACRω
		notifyIcon.ChangeIcon( m_hWnd, GetKeyState( VK_KANA ) & 1 ? IDI_ICON_ON : IDI_ICON_APP );
	}
	if ( shared->b.m.bVjed4 ) {
		//SendMessage( GetForegroundWindow(), wmDeltaEnd, 0, 0 );
		SendMessageTimeout( GetForegroundWindow(), wmDeltaEnd, 0, 0, SMTO_ABORTIFHUNG | SMTO_NORMAL, 1000, &dwResult );
	}
}

void MainWindow::Cls_OnSharedStatus( WPARAM wParam, LPARAM lParam )
{
	TCHAR temp[256];

	switch( wParam )
	{
	case STATUS_RUNMODE:
		switch( lParam )
		{
		case RUN_MODE_FALLEN:
			PostMessage( m_hWnd, WM_CLOSE, 0, 0 );
			break;
		}
		break;

	case STATUS_DENYSAME:
		MessageBox( m_hWnd, _T("vbgtH[̃X[uڑp܂"), szTitle, MB_OK | MB_ICONEXCLAMATION );
		break;

	case STATUS_DENYMANY:
		MessageBox( m_hWnd, _T("ő吔𒴂X[uڑp܂"), szTitle, MB_OK | MB_ICONEXCLAMATION );
		break;

	case STATUS_LAUNCH:
		_stprintf_s( temp, _countof(temp), _T("X[uvZXċN܂\r\n : %d / s : %d"), LOWORD(lParam), HIWORD(lParam) );
		MessageBox( m_hWnd, temp, szTitle, MB_OK | MB_ICONEXCLAMATION );
		break;

	case STATUS_RECONNECT:
		MessageBox( m_hWnd, _T("}X^[vZX̍ċNo܂"), szTitle, MB_OK | MB_ICONEXCLAMATION );
		break;

	case STATUS_REUSE:
		MessageBox( m_hWnd, _T("p̎c[𗬗p܂"), szTitle, MB_OK | MB_ICONEXCLAMATION );
		break;

	case STATUS_MASTERLOST:
		if ( !waitForMsgBox ) {
			waitForMsgBox = TRUE;
			if ( MessageBox( m_hWnd, _T("}X^[vZXr܂\r\nI܂"), szTitle, MB_YESNO | MB_ICONQUESTION ) == IDYES ) {
				PostMessage( m_hWnd, WM_CLOSE, 0, 0 );
			}
			waitForMsgBox = FALSE;
		}
		break;

	case STATUS_SLAVELOST:
		_stprintf_s( temp, _countof(temp), _T("%d ̃X[uvZXr܂"), lParam );
		MessageBox( m_hWnd, temp, szTitle, MB_OK | MB_ICONEXCLAMATION );
		break;

	case STATUS_BADDEL:
		_stprintf_s( temp, _countof(temp), _T("X[uȊO̍폜ʒmM܂ : %x"), lParam );
		MessageBox( m_hWnd, temp, szTitle, MB_OK | MB_ICONEXCLAMATION );
		break;

	case STATUS_ACCEPTED:
		Init();
		break;
	}
}
