#include "StdAfx.h"
#include "NotifyIcon.h"
#include "DynamicApi.h"
#include <shellapi.h>


const UINT WM_TASKBARCREATED = RegisterWindowMessage( _T("TaskbarCreated") );



NotifyIcon::NotifyIcon(void)
	: cbSize( 0 )
	, tipLen( 0 )
	, infoSupported( FALSE )
{
}



NotifyIcon::~NotifyIcon(void)
{
	if ( GetSimpleOSVersion() >= OS_VISTA ) {
		user32.ChangeWindowMessageFilter( WM_TASKBARCREATED, MSGFLT_REMOVE );
	}
}



BOOL NotifyIcon::Init(void)
{
	BOOL bRet = FALSE;
	DLLVERSIONINFO dvi = {sizeof( DLLVERSIONINFO )};
	ULONGLONG shellVer;

	do {
		if ( GetSimpleOSVersion() >= OS_VISTA ) {
			if ( !user32.ChangeWindowMessageFilter( WM_TASKBARCREATED, MSGFLT_ADD ) ) break;
		}

		if ( shell32.DllGetVersion( &dvi ) == E_NOTIMPL ) {
			cbSize = NOTIFYICONDATA_V1_SIZE;
		}
		else {
			shellVer = MAKEDLLVERULL( dvi.dwMajorVersion, dvi.dwMinorVersion, dvi.dwBuildNumber, 0 );

			if ( shellVer >= MAKEDLLVERULL( 6, 0, 6, 0 ) ) {
				cbSize = sizeof(NOTIFYICONDATA);
			}
			else if ( shellVer >= MAKEDLLVERULL( 6, 0, 0, 0 ) ) {
				cbSize = NOTIFYICONDATA_V3_SIZE;
			}
			else if ( shellVer >= MAKEDLLVERULL( 5, 0, 0, 0 ) ) {
				cbSize = NOTIFYICONDATA_V2_SIZE;
			}
			else {
				cbSize = NOTIFYICONDATA_V1_SIZE;
			}
		}

		if ( cbSize == NOTIFYICONDATA_V1_SIZE ) {
			tipLen = 64;
			infoSupported = FALSE;
		}
		else {
			tipLen = 128;
			infoSupported = TRUE;
		}

		bRet = TRUE;

	} while( FALSE );

	return bRet;
}



BOOL NotifyIcon::AddIcon( HWND hWnd, TCHAR *tip, UINT idResource, UINT id, UINT msg, HINSTANCE inst )
{
	NOTIFYICONDATA notif;
	BOOL bRet;
	int retry = 50;

	memset( &notif, 0, sizeof(NOTIFYICONDATA) );

	notif.hIcon = (HICON)LoadImage( inst, MAKEINTRESOURCE(idResource), IMAGE_ICON, GetSystemMetrics( SM_CXSMICON ), GetSystemMetrics( SM_CYSMICON ), LR_DEFAULTCOLOR );
	if ( notif.hIcon == NULL ) {
		notif.hIcon = LoadIcon( inst, MAKEINTRESOURCE(idResource) );
	}

	notif.cbSize = cbSize;
	notif.hWnd = hWnd;
	notif.uID = id;
	notif.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	notif.uCallbackMessage = msg;
	_tcscpy_s(notif.szTip, tipLen, tip );

	bRet = Shell_NotifyIcon( NIM_ADD, &notif );

	if ( !bRet ) {
		if ( !Shell_NotifyIcon( NIM_MODIFY, &notif ) ) {
			do {
				Sleep( 2000 );
			} while( !( bRet = Shell_NotifyIcon( NIM_ADD, &notif ) ) && ( retry-- > 0 ) );
		}
		else {
			bRet = TRUE;
		}
	}

	if ( GetSimpleOSVersion() >= OS_2000 ) {
		notif.uVersion = NOTIFYICON_VERSION;
		Shell_NotifyIcon( NIM_SETVERSION, &notif );
	}

	DestroyIcon( notif.hIcon );

	return bRet;
}



BOOL NotifyIcon::DeleteIcon( HWND hWnd, UINT id )
{
	NOTIFYICONDATA notif;
	BOOL bRet;

	memset( &notif, 0, sizeof(NOTIFYICONDATA) );

	notif.cbSize = cbSize;
	notif.hWnd = hWnd;
	notif.uID = id;

	bRet = Shell_NotifyIcon( NIM_DELETE, &notif );

	return bRet;
}

BOOL NotifyIcon::SetInfo(  HWND hWnd, TCHAR *info, UINT id )
{
	NOTIFYICONDATA notif;
	BOOL bRet;

	if ( !infoSupported ) return FALSE;

	memset( &notif, 0, sizeof(NOTIFYICONDATA) );

	notif.cbSize = cbSize;
	notif.hWnd = hWnd;
	notif.uID = id;
	notif.uFlags = NIF_INFO;
	_tcscpy_s( notif.szInfo, _countof(notif.szInfo), info );

	bRet = Shell_NotifyIcon( NIM_MODIFY, &notif );

	return bRet;
}

BOOL NotifyIcon::ChangeIcon( HWND hWnd, UINT idResource, UINT id, HINSTANCE inst )
{
	NOTIFYICONDATA notif;
	BOOL bRet;

	memset( &notif, 0, sizeof(NOTIFYICONDATA) );

	notif.hIcon = (HICON)LoadImage( inst, MAKEINTRESOURCE(idResource), IMAGE_ICON, GetSystemMetrics( SM_CXSMICON ), GetSystemMetrics( SM_CYSMICON ), LR_DEFAULTCOLOR );
	if ( notif.hIcon == NULL ) {
		notif.hIcon = LoadIcon( inst, MAKEINTRESOURCE(idResource) );
	}

	notif.cbSize = cbSize;
	notif.hWnd = hWnd;
	notif.uID = id;
	notif.uFlags = NIF_ICON;

	bRet = Shell_NotifyIcon( NIM_MODIFY, &notif );

	DestroyIcon( notif.hIcon );

	return bRet;
}
