#include "stdafx.h"
#include <stddef.h>
#include "SyncRun.h"
#include "misc.h"
#include "common.h"
#include "DynamicApi.h"


SyncRun::SyncRun(BOOL allowSameArchitechture, BOOL autoLanuchSlave)
	: mainWnd( NULL )
	, masterWnd( NULL )
	, sharedSize( 0 )
	, runMode( RUN_MODE_SINGLE )
	, timerID( 0 )
	, waitForSlaveClose( FALSE )
	, allowSameArch( allowSameArchitechture )
	, maxSlave( 1 )
	, slaveFinishedEvent( NULL )
	, sharedContext()
	, autoLaunch( autoLanuchSlave )
	, timerInterval( WATCH_TIMER_INTERVAL )
{
	m_lpszClass = _T("SyncRunDummyWindowClass");
	m_hbrBackground = NULL;
	m_style = WS_POPUP;
	m_exStyle = WS_EX_TOOLWINDOW;

	memset( sharedIdent, 0, _countof(sharedIdent) );
	memset( slavePath, 0, _countof(slavePath) );
	InitializeCriticalSection( &criticalSection );

	isNative = IsNativePlatform();

	if ( autoLaunch ) {
		if ( !allowSameArch && !Isx86() ) {

			TCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];
			TCHAR *p;

			GetModuleFileName( NULL, slavePath, _countof(slavePath) );
			_tsplitpath_s( slavePath, drive, _countof(drive), dir, _countof(dir), fname, _countof(fname), ext, _countof(ext) );

			if ( ( _tcslen( fname ) > 2 ) && ( _tcscmp( ( p = fname + _tcslen(fname) - 2 ), _T("64") ) == 0 ) ) {
					*p = 0;
			}
			else {
				_tcscat_s( fname, _countof(fname), _T("64") );
			}
			_tmakepath_s( slavePath, _countof(slavePath), drive, dir, fname, ext );
		}
		else if ( allowSameArch ) {
			GetModuleFileName( NULL, slavePath, _countof(slavePath) );
		}
	}
}

SyncRun::~SyncRun(void)
{
	if ( runMode == RUN_MODE_SLAVE ) {
		PostMessage( (HWND)sharedContext.pointer->masterWnd, WM_SHARED_HWND, SHARED_HWND_DEL, (WPARAM)m_hWnd );
	}
	CloseSharedMemory( &sharedContext );
	Exit();
	if ( slaveFinishedEvent ) CloseHandle( slaveFinishedEvent );
	DeleteCriticalSection( &criticalSection );

	if ( GetSimpleOSVersion() >= OS_VISTA ) {
		user32.ChangeWindowMessageFilter( WM_NULL,			MSGFLT_REMOVE );
		user32.ChangeWindowMessageFilter( WM_SHARED_UPDATE,	MSGFLT_REMOVE );
		user32.ChangeWindowMessageFilter( WM_SHARED_HWND,	MSGFLT_REMOVE );
		user32.ChangeWindowMessageFilter( WM_SHARED_CLOSE,	MSGFLT_REMOVE );
	}
}

UINT SyncRun::Init( HWND hWnd, TCHAR *ident, DWORD size, void **address, UINT numMaxSlave, UINT interval )
{
	BOOL bRet = FALSE;
	void *returnAddress = NULL;
	UINT retCode = INIT_ERR_CHGMSGFILTR;

	sharedSize = size;
	mainWnd = hWnd;
	maxSlave = numMaxSlave;

	if ( ident ) {
		_tcscpy_s( sharedIdent, _countof(sharedIdent), ident );
	}

	do {
		if ( GetSimpleOSVersion() >= OS_VISTA ) {
			if ( !user32.ChangeWindowMessageFilter( WM_NULL,			MSGFLT_ADD ) ) break;
			if ( !user32.ChangeWindowMessageFilter( WM_SHARED_UPDATE,	MSGFLT_ADD ) ) break;
			if ( !user32.ChangeWindowMessageFilter( WM_SHARED_HWND,		MSGFLT_ADD ) ) break;
			if ( !user32.ChangeWindowMessageFilter( WM_SHARED_CLOSE,	MSGFLT_ADD ) ) break;
		}

		returnAddress = OpenSharedMemory( &sharedContext, ident );
		if ( ( returnAddress == NULL ) || ( !IsWindowAlive( (HWND)sharedContext.pointer->masterWnd ) ) ) {

			retCode = INIT_ERR_CREATEEVENT;
			slaveFinishedEvent = CreateEvent( NULL, FALSE, FALSE, NULL );
			if ( slaveFinishedEvent == NULL ) break;

			if ( returnAddress == NULL ) {

				sharedContext.handle = CreateFileMapping(
						INVALID_HANDLE_VALUE,
						NULL,
						PAGE_READWRITE,
						0,
						sharedSize + offsetof( struct _SHARED_STRUCT, customStruct ),
						sharedIdent );

				retCode = INIT_ERR_CREATEFILEMAP;
				if ( sharedContext.handle == NULL ) {
					break;
				}

				retCode = INIT_ERR_MAPVIEWFILE;
				sharedContext.pointer = (SHARED_STRUCT*)MapViewOfFile( sharedContext.handle, FILE_MAP_ALL_ACCESS, 0, 0, 0 );
				if ( sharedContext.pointer == NULL ) {
					break;
				}
			}
			else {
				retCode = INIT_ERR_OCCUPYMEM;
				CloseSharedMemory( &sharedContext );
				returnAddress = OccupySharedMemory( &sharedContext, ident );

				if ( returnAddress == NULL ) {
					break;
				}
				else {
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_REUSE, 0 );
				}
			}

			memset( sharedContext.pointer, 0, sharedSize + offsetof( struct _SHARED_STRUCT, customStruct ) );

			runMode = RUN_MODE_SINGLE;

			returnAddress = GetSharedAddress( sharedContext.pointer );
		}
		else {

			runMode = RUN_MODE_SLAVE;

			masterWnd = (HWND)sharedContext.pointer->masterWnd;
		}

		retCode = INIT_ERR_CREATETHREAD;
		if ( !Create() ) break;

		bRet = TRUE;

	} while( FALSE );

	if ( !bRet ) {
		CloseSharedMemory( &sharedContext );
	}
	else {
		*address = returnAddress;
		retCode = INIT_ERR_SUCCESS;
		PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_RUNMODE, (LPARAM)runMode );
	}

	return retCode;
}

INT_PTR CALLBACK SyncRun::WndProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	UINT ref;
	HWND connectedWnd;
	BOOL bDeny = FALSE;

	switch( uMsg )
	{
	case WM_SHARED_STATUS:
	case WM_SHARED_UPDATE:
		PostMessage( mainWnd, uMsg, wParam, lParam );
		break;

	case WM_CREATE:
		timerID = SetTimer( m_hWnd, WATCH_TIMER_ID, timerInterval, NULL );
		if ( runMode == RUN_MODE_SLAVE ) {
			PostMessage( masterWnd, WM_SHARED_HWND, SHARED_HWND_ADD, (WPARAM)m_hWnd );
		}
		else {
			sharedContext.pointer->masterWnd = (DWORD)m_hWnd;
			if ( ( autoLaunch ) && ( slavePath[0] != 0 ) ) {
				if ( LaunchApp( slavePath ) ) {
					runMode = RUN_MODE_MASTER;
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_RUNMODE, (LPARAM)runMode );
				}
				else {
					runMode = RUN_MODE_SINGLE;
				}
			}
		}
		break;

	case WM_CLOSE:
		DestroyWindow( m_hWnd );
		break;

	case WM_DESTROY:
		PostQuitMessage(0);
		break;

	case WM_SHARED_HWND:

		connectedWnd = (HWND)lParam;

		if ( runMode != RUN_MODE_SLAVE ) {

			switch( wParam )
			{
			case SHARED_HWND_ADD:

				if ( !allowSameArch && ( IsWindowNativePlatform( connectedWnd ) == isNative ) ) {
					PostMessage( connectedWnd, WM_SHARED_CLOSE, CLOSE_PLATFORM, 0 );
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_DENYSAME, 0 );
				}
				else {
					EnterCriticalSection( &criticalSection );

					if ( slaveWnd.size() >= maxSlave ) {
						bDeny = TRUE;
					}
					else {
						slaveWnd.push_back( connectedWnd );
					}
					ref = (UINT)slaveWnd.size();

					LeaveCriticalSection( &criticalSection );

					if ( bDeny ) {
						PostMessage( connectedWnd, WM_SHARED_CLOSE, CLOSE_TOOMANY, 0 );
						PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_DENYMANY, 0 );
					}
					else {
						if ( runMode != RUN_MODE_MASTER ) {
							runMode = RUN_MODE_MASTER;
							PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_RUNMODE, (LPARAM)runMode );
						}
						PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_SLAVENUM, (LPARAM)ref );
						PostMessage( connectedWnd, WM_SHARED_STATUS, STATUS_ACCEPTED, 0 );
					}
				}
				break;

			case SHARED_HWND_DEL:

				EnterCriticalSection( &criticalSection );

				if ( slaveWnd.size() == 0 ) {
					bDeny = TRUE;
				}
				else {
					BOOL deleted;
					DeleteSlave( connectedWnd, &deleted );
					bDeny = !deleted;
				}
				ref = (UINT)slaveWnd.size();

				LeaveCriticalSection( &criticalSection );

				if ( bDeny == TRUE ) {
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_BADDEL, (LPARAM)connectedWnd );
				}
				else {
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_SLAVENUM, (LPARAM)ref );

					if ( ( ref == 0 ) && ( runMode != RUN_MODE_SINGLE ) ) {
						runMode = RUN_MODE_SINGLE;
						PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_RUNMODE, (LPARAM)runMode );
						if ( waitForSlaveClose ) {
							SetEvent( slaveFinishedEvent );
						}
					}
				}
				break;
			}
			return TRUE;
		}
		break;

	case WM_SHARED_CLOSE:
		if ( wParam != CLOSE_NORMAL ) runMode = RUN_MODE_FALLEN;
		PostMessage( mainWnd, WM_CLOSE, 0, 0 );
		return TRUE;

	case WM_TIMER:

		if ( wParam == WATCH_TIMER_ID ) {

			if ( runMode == RUN_MODE_SLAVE ) {

				if ( ( masterWnd != (HWND)sharedContext.pointer->masterWnd ) &&
				 IsWindowAlive( (HWND)sharedContext.pointer->masterWnd ) &&
				 ( slaveWnd.size() < maxSlave ) ) {

					masterWnd = (HWND)sharedContext.pointer->masterWnd;
					PostMessage( masterWnd, WM_SHARED_HWND, SHARED_HWND_ADD, (WPARAM)m_hWnd );
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_RECONNECT, 0 );
				}
				else if ( !IsWindowAlive( masterWnd ) ) {
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_MASTERLOST, 0 );
				}
			}
			else {
				UINT numSucceeded = 0, numFailed = 0, ref0, ref;
				std::vector<HWND>::iterator Iter;

				if ( sharedContext.pointer->masterWnd != (DWORD)m_hWnd ) {
					m_bLoop = FALSE;
					runMode = RUN_MODE_FALLEN;
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_RUNMODE, (LPARAM)runMode );
					break;
				}

				EnterCriticalSection( &criticalSection );

				ref0 = (UINT)slaveWnd.size();

				Iter = slaveWnd.begin();
				while( Iter != slaveWnd.end() ) {

					if ( !IsWindowAlive( (*Iter) ) ) {

						KillWindowProcess( (*Iter) );
						Iter = DeleteSlave( (*Iter) );

						if ( ( autoLaunch ) && ( slavePath[0] != 0 ) ) {
							if ( LaunchApp( slavePath ) ) {
								numSucceeded++;
								runMode = RUN_MODE_MASTER;
							}
						}
						else {
							numFailed++;
						}
						continue;
					}
					Iter++;
				}
				ref = (UINT)slaveWnd.size();

				LeaveCriticalSection( &criticalSection );

				if ( ( ref == 0 ) && ( runMode != RUN_MODE_SINGLE ) ) {
					runMode = RUN_MODE_SINGLE;
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_RUNMODE, (LPARAM)runMode );
				}

				if ( ( numSucceeded > 0 ) || ( numFailed > 0 ) ) {
					if ( autoLaunch ) {
						PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_LAUNCH, MAKELPARAM( numSucceeded, numFailed ) );
					}
					else {
						PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_SLAVELOST, (LPARAM)numFailed );
					}
				}

				if ( ref != ref0 ) {
					PostMessage( mainWnd, WM_SHARED_STATUS, STATUS_SLAVENUM, (LPARAM)ref );
				}
			}
			return TRUE;
		}
		break;
	}
	return 0;
}

BOOL SyncRun::SharedUpdate(void)
{
	BOOL bRet = TRUE;
	std::vector<HWND>::iterator Iter;

	if ( runMode == RUN_MODE_MASTER ) {

		EnterCriticalSection( &criticalSection );

		for ( Iter = slaveWnd.begin(); Iter != slaveWnd.end(); Iter++ ) {
			if ( !PostMessage( (*Iter), WM_SHARED_UPDATE, 0, 0 ) ) {
				bRet = FALSE;
			}
		}

		LeaveCriticalSection( &criticalSection );
	}
	return bRet;
}

void SyncRun::CloseApp( HWND hWnd, UINT timeout )
{
	std::vector<HWND>::iterator Iter;

	if ( timerID ) KillTimer( hWnd, timerID );
	timerID = 0;

	switch( runMode )
	{
	case RUN_MODE_MASTER:

		if ( slaveWnd.size() > 0 ) {

			waitForSlaveClose = TRUE;
			size_t numSlave = slaveWnd.size();
			Iter = slaveWnd.begin();
			while( Iter != slaveWnd.end() ) {
				if ( IsWindowAlive( (*Iter) ) ) {
					PostMessage( (*Iter), WM_SHARED_CLOSE, CLOSE_NORMAL, 0 );
				}
				else {
					KillWindowProcess( (*Iter) );
					Iter = DeleteSlave( (*Iter) );
					continue;
				}
				Iter++;
			}
			if ( numSlave > 0 ) {
				DWORD dwRet = WaitForSingleObject( slaveFinishedEvent, timeout );
				if ( dwRet == WAIT_TIMEOUT ) {
					Iter = slaveWnd.begin();
					while( Iter != slaveWnd.end() ) {
						KillWindowProcess( (*Iter) );
						Iter++;
					}
				}
			}
		}
		Exit();
		DestroyWindow( hWnd );
		break;

	default:
		Exit();
		DestroyWindow( hWnd );
		break;
	}
}

std::vector<HWND>::iterator SyncRun::DeleteSlave( HWND hWnd, BOOL *deleted )
{
	std::vector<HWND>::iterator Iter;

	if ( deleted ) *deleted = FALSE;
	Iter = slaveWnd.begin();
	while( Iter != slaveWnd.end() ) {
		if ( (*Iter) == hWnd ) {
			Iter = slaveWnd.erase( Iter );
			if ( deleted ) *deleted = TRUE;
			break;
		}
		Iter++;
	}

	return Iter;
}

BOOL SyncRun::Exit( DWORD dwTimeout )
{
	m_bLoop = FALSE;
	PostMessage( m_hWnd, WM_CLOSE, 0, 0 );
	return Join( dwTimeout );
}

BOOL SyncRun::Create(void)
{
	return Thread::Create();
}

DWORD SyncRun::ThreadProc(void)
{
	Window::Create( _T("SyncRunDummyWindow"), 10, 10, 10, 10, SW_HIDE );
	return (DWORD)RunMsgLoop();
}
