#ifndef _SYNCRUN_H_
#define _SYNCRUN_H_

#include "Thread.h"
#include "Window.h"
#include "ShareMem.h"
#include <vector>

#define RUN_MODE_SINGLE	0
#define RUN_MODE_MASTER	1
#define RUN_MODE_SLAVE	2
#define RUN_MODE_FALLEN	3

#define INIT_ERR_SUCCESS		0
#define INIT_ERR_CREATEEVENT	1
#define INIT_ERR_CHGMSGFILTR	2
#define INIT_ERR_CREATEFILEMAP	3
#define INIT_ERR_MAPVIEWFILE	4
#define INIT_ERR_OCCUPYMEM		5
#define INIT_ERR_CREATETHREAD	6

#define WM_SHARED_UPDATE	(WM_APP+100)
#define WM_SHARED_STATUS	(WM_APP+101)
#define STATUS_RUNMODE		0
#define STATUS_SLAVENUM		1
#define STATUS_DENYSAME		2
#define STATUS_DENYMANY		3
#define STATUS_LAUNCH		4
#define STATUS_RECONNECT	5
#define STATUS_REUSE		6
#define STATUS_MASTERLOST	7
#define STATUS_SLAVELOST	8
#define STATUS_BADDEL		9
#define STATUS_ACCEPTED		10
#define WM_SHARED_HWND		(WM_APP+102)
#define SHARED_HWND_ADD		0
#define SHARED_HWND_DEL		1
#define WM_SHARED_CLOSE		(WM_APP+103)
#define CLOSE_NORMAL		0
#define CLOSE_PLATFORM		1
#define CLOSE_TOOMANY		2

#define WATCH_TIMER_ID			53248
#define WATCH_TIMER_INTERVAL	10000
#define TERMINATE_TIMEOUT		5000

class SyncRun : public Window, public Thread
{
public:
	SyncRun(BOOL allowSameArchitechture = FALSE, BOOL autoLanuchSlave = TRUE);
	virtual ~SyncRun(void);

	UINT Init( HWND hWnd, TCHAR *ident, DWORD size, void **address, UINT numMaxSlave = 1, UINT interval = WATCH_TIMER_INTERVAL );
	INT_PTR CALLBACK TranslateMessage( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
	BOOL SharedUpdate(void);
	void CloseApp( HWND hWnd, UINT timeout = TERMINATE_TIMEOUT );
	UINT GetRunMode(void) { return runMode; }
	UINT GetSlaveCount(void) { return (UINT)slaveWnd.size(); };
	BOOL IsSlaveMode(void) { return runMode == RUN_MODE_SLAVE; };

	DWORD ThreadProc(void);
	BOOL Create(void);
	INT_PTR CALLBACK WndProc( UINT uMsg, WPARAM wParam, LPARAM lParam );

protected:
	HWND mainWnd, masterWnd;
	BOOL isNative;
	DWORD sharedSize;
	SHARED_CONTEXT sharedContext;
	TCHAR sharedIdent[256];
	std::vector<HWND> slaveWnd;
	UINT runMode;
	UINT_PTR timerID;
	BOOL waitForSlaveClose;
	TCHAR slavePath[_MAX_PATH];
	BOOL allowSameArch;
	UINT maxSlave;
	HANDLE slaveFinishedEvent;
	CRITICAL_SECTION criticalSection;
	BOOL autoLaunch;
	UINT timerInterval;

	std::vector<HWND>::iterator DeleteSlave( HWND hWnd, BOOL *deleted = NULL );
	virtual BOOL Exit( DWORD dwTimeout = INFINITE );
};

#endif//_SYNCRUN_H_
