#include "stdafx.h"
#include <windows.h>
#include "DynamicApi.h"


typedef HMODULE (WINAPI *LOADLIBRARY)(LPCTSTR);
typedef BOOL (WINAPI *FREELIBRARY)(HMODULE);
typedef BOOL (*DUMMYWINDOW)(void);
typedef FARPROC (WINAPI *GETPROCADDRESS)(HMODULE,LPCSTR);


typedef struct _ATTACH_CONTEXT {
	LOADLIBRARY pLoadLibrary;
	FREELIBRARY pFreeLibrary;
	GETPROCADDRESS pGetProcAddress;
	TCHAR path[_MAX_PATH];
	char functionName[100];
} ATTACH_CONTEXT;


#define BREAK_IF_NULL(x)	if((x)==NULL)break
#define BREAK_IF_ZERO(x)	if((x)==0)break
#define BREAK_IF_FALSE(x)	if(!(x))break

#define IS_OUT_OF_RANGE(x,y,z)	(((LPBYTE)x<(LPBYTE)y)||((LPBYTE)x>=((LPBYTE)y+z)))

static const DWORD PROCESS_ACCESS_RIGHTS = PROCESS_CREATE_THREAD | PROCESS_VM_OPERATION | PROCESS_VM_READ | PROCESS_VM_WRITE | PROCESS_QUERY_INFORMATION;



extern "C" DWORD WINAPI AttachThreadProc( ATTACH_CONTEXT *context )
{
	HMODULE hmod;
	DUMMYWINDOW pDummyWindow = NULL;
	DWORD ret = 0;

	hmod = context->pLoadLibrary( context->path );

	if ( hmod ) {
		pDummyWindow = (DUMMYWINDOW)context->pGetProcAddress( hmod, context->functionName );
		if ( pDummyWindow ) {
			ret = (DWORD)pDummyWindow();
		}
		context->pFreeLibrary( hmod );
	}

	return ret;
}

extern "C" void AfterProcPlaceHolder(void) {
}



BOOL EjectDll( DWORD processId, ATTACH_CONTEXT *localContext )
{
	BOOL bRet = FALSE;
	HANDLE process, thread;
	ATTACH_CONTEXT *targetContext = NULL;
	LPVOID targetCode = NULL;
	DWORD threadId, exitCode;
	SIZE_T codeSize;

	do {
		BREAK_IF_NULL( process = OpenProcess( PROCESS_ACCESS_RIGHTS, FALSE, processId ) );

		BREAK_IF_NULL( targetContext = (ATTACH_CONTEXT*)VirtualAllocEx( process, NULL, sizeof(ATTACH_CONTEXT), MEM_COMMIT, PAGE_READWRITE ) );
		BREAK_IF_ZERO( WriteProcessMemory( process, targetContext, localContext, sizeof(ATTACH_CONTEXT), NULL ) );

		codeSize = (LPBYTE)AfterProcPlaceHolder - (LPBYTE)AttachThreadProc;
		BREAK_IF_NULL( targetCode = VirtualAllocEx( process, NULL, codeSize, MEM_COMMIT, PAGE_EXECUTE_READWRITE ) );
		BREAK_IF_ZERO( WriteProcessMemory( process, targetCode, AttachThreadProc, codeSize, NULL ) );

		BREAK_IF_NULL( thread = CreateRemoteThread( process, NULL, 0, (LPTHREAD_START_ROUTINE)targetCode, targetContext, 0, &threadId ) );
		WaitForSingleObject( thread, INFINITE );

		BREAK_IF_ZERO( GetExitCodeThread( thread, &exitCode ) );
		bRet = ( exitCode != 0 ) ? TRUE : FALSE;

	} while ( FALSE );

	if ( targetCode    ) VirtualFreeEx( process, targetCode,	0, MEM_RELEASE );
	if ( targetContext ) VirtualFreeEx( process, targetContext, 0, MEM_RELEASE );
	if ( thread  ) CloseHandle( thread	);
	if ( process ) CloseHandle( process );

	return bRet;
}



BOOL GetRealProcAddress( HMODULE moduleHandle, const char *procName, FARPROC &procAddress )
{
	BOOL bRet = FALSE;
	MODULEINFO moduleInfo = {0};
	FARPROC tempAddr;

	do {
		BREAK_IF_FALSE( psapi.GetModuleInformation( GetCurrentProcess(), moduleHandle, &moduleInfo, sizeof(moduleInfo) ) );

		tempAddr = GetProcAddress( moduleHandle, procName );

		if ( IS_OUT_OF_RANGE( tempAddr, moduleInfo.lpBaseOfDll, moduleInfo.SizeOfImage ) ) {

			NtDllApi ntDll;
			ANSI_STRING strGetProcAddr;

			BREAK_IF_FALSE( ntDll.Init() );
			ntDll.RtlInitAnsiString( &strGetProcAddr, procName );
			ntDll.LdrGetProcedureAddress( moduleHandle, &strGetProcAddr, 0, &tempAddr );

			if ( IS_OUT_OF_RANGE( tempAddr, moduleInfo.lpBaseOfDll, moduleInfo.SizeOfImage ) ) break;
		}

		bRet = TRUE;

	} while ( FALSE );

	if ( bRet ) procAddress = tempAddr;

	return bRet;
}



BOOL InitAttachContext( ATTACH_CONTEXT *context, const TCHAR *path )
{
	BOOL bRet = FALSE;
	HMODULE kernel32;
	const char *nameLoadLibrary;

#ifdef _UNICODE
	nameLoadLibrary = "LoadLibraryW";
#else
	nameLoadLibrary = "LoadLibraryA";
#endif

	kernel32 = GetModuleHandle( _T("kernel32.dll") );

	do {
		BREAK_IF_FALSE( GetRealProcAddress( kernel32, nameLoadLibrary,	(FARPROC&)context->pLoadLibrary ) );
		BREAK_IF_FALSE( GetRealProcAddress( kernel32, "FreeLibrary",	(FARPROC&)context->pFreeLibrary ) );
		BREAK_IF_FALSE( GetRealProcAddress( kernel32, "GetProcAddress",	(FARPROC&)context->pGetProcAddress ) );
		_tcscpy_s( context->path, _countof(context->path), path );
		strcpy_s( context->functionName, _countof(context->functionName), "DummyWindow" );
		bRet = TRUE;
	} while ( FALSE );

	return bRet;
}



BOOL IsModuleLoaded( DWORD processId, const TCHAR *name, BOOL fullPath = FALSE )
{
	HANDLE processHandle = NULL;
	HMODULE *arrayModules = NULL;
	DWORD sizeArray, sizeNeeded;
	BOOL bRet = FALSE;
	TCHAR path[_MAX_PATH], *fname;

	do {
		processHandle = OpenProcess( PROCESS_ACCESS_RIGHTS, FALSE, processId );
		BREAK_IF_NULL( processHandle );

		sizeArray = 100 * sizeof(DWORD);
		arrayModules = new HMODULE[sizeArray / sizeof(DWORD)];

		BREAK_IF_FALSE( psapi.EnumProcessModules( processHandle, arrayModules, sizeArray, &sizeNeeded ) );

		if ( sizeArray < sizeNeeded ) {
			delete[] arrayModules;
			sizeArray = sizeNeeded;
			arrayModules = new HMODULE[sizeArray / sizeof(DWORD)];
			BREAK_IF_FALSE( psapi.EnumProcessModules( processHandle, arrayModules, sizeArray, &sizeNeeded ) );
		}

		sizeNeeded /= sizeof(DWORD);
		for ( DWORD c = 0; c < sizeNeeded; c++ ) {

			psapi.GetModuleFileNameEx( processHandle, arrayModules[c], path, _MAX_PATH );
			fname = fullPath ? path : ( _tcsrchr( path, _T('\\') ) + 1 );

			if ( _tcsicmp( fname, name ) == 0 ) {
				bRet = TRUE;
				break;
			}
		}
	} while ( FALSE );

	if ( arrayModules ) delete[] arrayModules;
	CloseHandle( processHandle );

	return bRet;
}



DWORD GetProcessList( DWORD **processArray, BOOL onlySameArch = FALSE )
{
	DWORD numProcess = 0, enumedProcess;

	*processArray = NULL;

	do {
		if ( *processArray ) delete[] *processArray;

		numProcess += 100;
		*processArray = new DWORD[numProcess];

		psapi.EnumProcesses( *processArray, numProcess * sizeof(DWORD), &enumedProcess );
		enumedProcess /= sizeof(DWORD);

	} while( enumedProcess >= numProcess );

	if ( onlySameArch ) {

		BOOL bThisIsWow64, bIsWow64;
		DWORD numSame;
		DWORD *tmpList;
		HANDLE process;

		kernel32.IsWow64Process( GetCurrentProcess(), &bThisIsWow64 );

		numSame = 0;
		tmpList = new DWORD[enumedProcess];

		numSame = 0;
		for ( DWORD c = 0; c < enumedProcess; c++ ) {

			process = OpenProcess( PROCESS_ACCESS_RIGHTS, FALSE, (*processArray)[c] );
			if ( process == NULL ) continue;

			kernel32.IsWow64Process( process, &bIsWow64 );
			if ( bIsWow64 == bThisIsWow64 ) {
				tmpList[numSame++] = (*processArray)[c];
			}
			CloseHandle( process );
		}

		delete[] *processArray;
		*processArray = tmpList;
		enumedProcess = numSame;
	}

	return enumedProcess;
}



DWORD UnloadWndProcHookDll( const TCHAR *dllNameToUnload )
{
	DWORD *processArray = NULL;
	DWORD enumedProcess;
	ATTACH_CONTEXT context;
	DWORD numUnloaded = 0;
	DWORD currentProcess = GetCurrentProcessId();

	do {
		BREAK_IF_FALSE( psapi.Init() );
		BREAK_IF_FALSE( InitAttachContext( &context, deltaDll.GetPath() ) );

		enumedProcess = GetProcessList( &processArray, TRUE );

		for ( DWORD c = 0; c < enumedProcess; c++ ) {
			if ( processArray[c] == currentProcess ) continue;
			if ( IsModuleLoaded( processArray[c], dllNameToUnload, TRUE ) ) {
				if ( EjectDll( processArray[c], &context ) ) numUnloaded++;
			}
		}
		delete[] processArray;

	} while ( FALSE );

	return numUnloaded;
}
