#include "stdafx.h"
#include <windows.h>
#include <shlwapi.h>
#include "misc.h"
#include "common.h"
#include "DynamicApi.h"

#define BREAK_IF_NULL(x)	if((x)==NULL)break
#define BREAK_IF_ZERO(x)	if((x)==0)break
#define BREAK_IF_FALSE(x)	if(!(x))break


BOOL LaunchApp( TCHAR *path, BOOL noui )
{
	SHELLEXECUTEINFO sei = {0};

	sei.cbSize = sizeof(SHELLEXECUTEINFO);
	sei.nShow = SW_NORMAL;
	sei.lpFile = path;
	sei.fMask = noui ? SEE_MASK_FLAG_NO_UI : 0;

	return ShellExecuteEx( &sei );
}

BOOL IsWindowAlive( HWND hWnd, DWORD *err )
{
	BOOL bRet = TRUE;
	DWORD dwErr;

	bRet = PostMessage( hWnd, WM_NULL, 0, 0 );

	if ( !bRet ) {
		dwErr = GetLastError();
		if ( dwErr == ERROR_INVALID_WINDOW_HANDLE ) {
			bRet = FALSE;
		}
		if ( err ) {
			*err = dwErr;
		}
	}

	return bRet;
}

BOOL IsWow64()
{
	BOOL bRet = FALSE;

	kernel32.IsWow64Process( GetCurrentProcess(), &bRet );

	return bRet;
}

BOOL Isx86()
{
	BOOL bRet = TRUE;
	SYSTEM_INFO si;

	do {
		if ( !kernel32.GetNativeSystemInfo( &si ) ) break;

		if ( si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL ) break;

		bRet = FALSE;

	} while( FALSE );

	return bRet;
}

HANDLE OpenWindowProcess( HWND hWnd, DWORD desiredAccess )
{
	DWORD threadId, processId;
	HANDLE processHandle = NULL;

	do {
		threadId = GetWindowThreadProcessId( hWnd, &processId );
		if ( threadId == 0 ) break;

		processHandle = OpenProcess( desiredAccess, FALSE, processId );

	} while ( FALSE );

	return processHandle;
}

BOOL IsWindowNativePlatform( HWND hWnd )
{
	BOOL bWow64 = FALSE;
	HANDLE processHandle;

	do {
		processHandle = OpenWindowProcess( hWnd, PROCESS_QUERY_LIMITED_INFORMATION );
		if ( processHandle == NULL ) break;

		kernel32.IsWow64Process( processHandle, &bWow64 );
		bWow64 = bWow64 ? FALSE : TRUE;
		CloseHandle( processHandle );

	} while ( FALSE );

	return bWow64;
}

BOOL KillWindowProcess( HWND hWnd )
{
	BOOL bRet = FALSE;
	HANDLE processHandle;

	do {
		processHandle = OpenWindowProcess( hWnd, PROCESS_TERMINATE );
		if ( processHandle == NULL ) break;

		bRet = TerminateProcess( processHandle, 0 );
		CloseHandle( processHandle );

	} while ( FALSE );

	return bRet;
}

void PostCompleteBroadcast( UINT msg, WPARAM wParam, LPARAM lParam )
{
	HWND hAfter = NULL, hWnd;

	PostMessage( HWND_BROADCAST, msg, wParam, lParam );

	while ( ( hWnd = FindWindowEx( HWND_MESSAGE, hAfter, NULL, NULL ) ) != NULL ) {
		PostMessage( hWnd, msg, wParam, lParam );
		hAfter = hWnd;
	};
}

BOOL GetProcessName( DWORD processId, TCHAR *dest, DWORD count )
{
	BOOL bRet = FALSE;
	HANDLE process;
	HMODULE module;
	DWORD dummy;
	TCHAR path[_MAX_PATH], *p;

	if ( ( dest == NULL ) || ( count == 0 ) ) return FALSE;

	do {
		BREAK_IF_NULL( process = OpenProcess( PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, processId ) );

		if ( GetSimpleOSVersion() <= OS_2000 ) {
			BREAK_IF_FALSE( psapi.EnumProcessModules( process, &module, sizeof(module), &dummy ) );
			BREAK_IF_ZERO( psapi.GetModuleFileNameEx( process, module, path, _countof(path) ) );
		}
		else {
			BREAK_IF_ZERO( psapi.GetProcessImageFileName( process, path, _countof(path) ) );
		}

		if ( _tcslen( path ) < 2 ) break;
		BREAK_IF_NULL( p = _tcsrchr( path, _T('\\') ) );

		_tcscpy_s( dest, count, p + 1 );
		bRet = TRUE;

	} while( FALSE );

	if ( process ) CloseHandle( process );

	if ( !bRet ) dest[0] = 0;

	return bRet;
}
